package medusa.georgios.ClusteringAlgorithms;
// The neighbour-joining algorithm.  Make a rooted tree by arbitrarily
// adding a root node with edges to the last two leaves

public class NJ {
  int N;			// The initial number of leaves
  int K;			// The number of clusters created so far
  NJCluster[] cluster;		// The nodes (clusters) of the resulting tree
  float[] r;			// The average distance to other leaves
  public StringBuffer NHX = new StringBuffer("");
  
  public NJ(float[][] ds) {
    N = ds.length;
    cluster = new NJCluster[2*N-1];
    for (int i=0; i<N; i++) 
      cluster[i] = new NJCluster(i, ds[i]);
    // System.out.println("Additive = " + checkAdditivity());
    r = new float[2*N-1];
    K = N;
    while (K < 2*N-2)
      findAndJoin();
    // Two leaves remain; cluster[K-1] is one of them, go find the other
    // Arbitrarily add a root node at this point
    int K2 = K-2;
    while (!cluster[K2].live())
      K2--;
    float dij = d(K2, K-1) / 2;
    // System.out.println("Joining " + K + "[" + dij + "] and " 
    //	       + (K2+1) + "[" + dij + "] to form " + (K+1));
    cluster[K] = new NJCluster(K, cluster[K2], dij, cluster[K-1], dij, null);
    K++;
  }

  void computeR() {
    for (int i=0; i<K; i++) 
      if (cluster[i].live()) {
	float sum = 0;
	for (int k=0; k<K; k++)
	  if (cluster[k].live() && k != i) 
	    sum += d(i, k);
	int L = 2 * N - K;	// The current number of leaves
	r[i] = sum / (L - 2);	// Strange, but the book says so (p 171)
      }
  }

  public NJCluster getRoot()
  { return cluster[K-1]; }
  
  public float d(int i, int j) 
  { return cluster[Math.max(i, j)].dmat[Math.min(i, j)]; }

  void findAndJoin() { // Find closest two live clusters and join them
    computeR();
    int mini = -1, minj = -1;
    float mind = Float.POSITIVE_INFINITY;
    for (int i=0; i<K; i++) 
      if (cluster[i].live())
	for (int j=0; j<i; j++) 
	  if (cluster[j].live()) {
	    float d = d(i, j) - (r[i] + r[j]);
	    if (d < mind) {
	      mind = d;
	      mini = i;
	      minj = j;
	    }
	  }
    join(mini, minj);
  }

  public void join(int i, int j) { // Join i and j to form node K
    float[] dmat = new float[K];
    float dij = d(i, j);
    for (int m=0; m<K; m++)
      if (cluster[m].live() && m != i && m != j) 
	dmat[m] = (d(i, m) + d(j, m) - dij) / 2;
    float dik = (dij + r[i] - r[j]) / 2;
    float djk = dij - dik;
    // System.out.println("Joining " + (i+1) + "[" + dik + "] and " 
    //	       + (j+1) + "[" + djk + "] to form " + (K+1));
    cluster[K] = new NJCluster(K, cluster[i], dik, cluster[j], djk, dmat);
    cluster[i].kill(); 
    cluster[j].kill();
    K++;
  }

  public boolean checkAdditivity() {
    for (int i=0; i<N; i++)
      for (int j=i+1; j<N; j++)
	for (int k=j+1; k<N; k++)
	  for (int m=k+1; m<N; m++) {
	    float dijdkm = d(i, j) + d(k, m);
	    float dikdjm = d(i, k) + d(j, m);
	    float dimdjk = d(i, m) + d(j, k);
	    if (!(dijdkm == dikdjm && dijdkm >= dimdjk
		  || dijdkm == dimdjk && dijdkm >= dikdjm
		  || dikdjm == dimdjk && dikdjm >= dijdkm)) {
	      System.out.println("(i, j, k, m) = ("+i+","+j+","+k+","+m+")");
	      return false;
	    }
	  }
    return true;
  }
  
  
  
   public boolean is_internal(NJCluster cluster)
    {
        if( cluster.left == null &&  cluster.right == null)
            return false;
        else
            return true;              
    }
  
   public String NewickFormat(NJCluster cluster) {
       if(this.is_internal(cluster)){
            NHX.append("(");
            this.NewickFormat(cluster.left);}
        
         if(this.is_internal(cluster)){NHX.append(",");}
        else
        {NHX.append(Clustering_NJ_UPGMA.labels.get(cluster.lab-1));}
        if(this.is_internal(cluster)){
            this.NewickFormat(cluster.right);
        NHX.append(")");
        } 
       return NHX.toString()+";";

  }
  
}
